﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Umc.Core;

namespace Sample.ButtonEx
{
	/// <summary>
	/// Before 이벤트가 발생할 때 전달될 이벤트 인자 클래스입니다.
	/// </summary>
	public class BeforeClickEventArgs : CancelEventArgs
	{
		public string Reason { get; set; }
	}

	/// <summary>
	/// After 이벤트가 발생할 때 전달될 이벤트 인자 클래스입니다.
	/// </summary>
	public class AfterClickEventArgs : EventArgs { }

	/// <summary>
	/// BeforeClick/AfterClick 이벤트가 추가된 버튼 서버 컨트롤 클래스 입니다.
	/// </summary>
	[DefaultProperty("Text")]
	[ToolboxData("<{0}:ButtonEx runat=\"server\"></{0}:ButtonEx>")]
	public class ButtonEx : Button, INamingContainer
	{
		/// <summary>
		/// BeforeClick 이벤트를 캡슐화 하는 델리게이트 입니다.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public delegate void BeforeClickEventHandler(object sender, BeforeClickEventArgs e);
		/// <summary>
		/// AfterClick 이벤트를 캡슐화 하는 델리게이트 입니다.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public delegate void AfterClickEventHandler(object sender, AfterClickEventArgs e);

		private EventHandlerDictionary events;
		/// <summary>
		/// EventHandlerDictionary 를 통해 이벤트를 관리합니다.
		/// </summary>
		protected EventHandlerDictionary Events
		{
			get
			{
				if( events == null )
					events = new EventHandlerDictionary();

				return events;
			}
		}

		private object EVENT_BEFORE_CLICK	= new object();	// BeforeClick 이벤트 키 오브젝트
		private object EVENT_AFTER_CLICK	= new object(); // AfterClick 이벤트 키 오브젝트

		/// <summary>
		/// BeforeClick 이벤트 입니다. Click 이벤트가 발생하기 전에 발생합니다.
		/// </summary>
		[Description("Click 이벤트가 발생하기 전에 발생합니다")]
		public event BeforeClickEventHandler BeforeClick
		{
			add
			{
				this.Events.AddHandler(EVENT_BEFORE_CLICK, value);
			}
			remove
			{
				this.Events.RemoveHandler(EVENT_BEFORE_CLICK, value);
			}
		}

		/// <summary>
		/// AfterClick 이벤트 입니다. Click 이벤트가 발생한 후 발생합니다.
		/// </summary>
		[Description("Click 이벤트가 발생 후에 발생합니다")]
		public event AfterClickEventHandler AfterClick
		{
			add
			{
				this.Events.AddHandler(EVENT_AFTER_CLICK, value);
			}
			remove
			{
				this.Events.RemoveHandler(EVENT_AFTER_CLICK, value);
			}
		}

		/// <summary>
		/// BeforeClick 이벤트를 발생합니다.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected virtual void OnBeforeClick(object sender, BeforeClickEventArgs e)
		{
			if( !this.Events.Contains( EVENT_BEFORE_CLICK ) ) return;

			BeforeClickEventHandler handler	= this.Events[ EVENT_BEFORE_CLICK ] as BeforeClickEventHandler;
			if( handler != null )
				handler(sender, e);
		}

		/// <summary>
		/// AfterClick 이벤트를 발생합니다.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		protected virtual void OnAfterClick(object sender, AfterClickEventArgs e)
		{
			if( !this.Events.Contains(EVENT_AFTER_CLICK) ) return;

			AfterClickEventHandler handler = this.Events[ EVENT_AFTER_CLICK ] as AfterClickEventHandler;
			if( handler != null )
				handler(sender, e);
		}

		/// <summary>
		/// Click 이벤트가 발생할 경우 BeforeClick/AfterClick 이벤트를 발생합니다.
		/// </summary>
		/// <param name="sender"></param>
		protected virtual void OnEventsFire( object sender )
		{
			BeforeClickEventArgs beforeArgs	= new BeforeClickEventArgs();
			OnBeforeClick( sender, beforeArgs );

			if( beforeArgs.Cancel ) return;

			AfterClickEventArgs afterArgs	= new AfterClickEventArgs();
			OnAfterClick( sender, afterArgs );
		}

		[Obsolete("ButtonEx 컨트롤에서는 Click 이벤트를 사용하지 않도록 합니다. 대신 AfterClick 이벤트를 사용하세요")]
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);
			this.Click += new EventHandler(ServerControl1_Click);
		}

		private void ServerControl1_Click(object sender, EventArgs e)
		{
			OnEventsFire(sender);
		}

	}
}
